import { Injectable } from '@nestjs/common';
import {
  EventTopics,
  columnUniques,
  validateRelations,
} from 'src/core/strings/constants/interface.constants';
import { BaseCreateManager } from 'src/core/modules/domain/usecase/managers/base-create.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { QueueItemModel, QueueModel } from '../../data/models/queue.model';
import { padCode } from 'src/modules/transaction/vip-code/domain/usecases/managers/helpers/generate-random.helper';
import { QueueBucketReadService } from '../../data/services/queue-bucket';
import { ItemModel } from 'src/modules/item-related/item/data/models/item.model';

@Injectable()
export class RegisterQueueManager extends BaseCreateManager<QueueModel> {
  constructor(private readonly bucketService: QueueBucketReadService) {
    super();
  }

  async averageTime(): Promise<number> {
    const item = await this.getItemMaster();
    return item.play_estimation;
  }

  async beforeProcess(): Promise<void> {
    const vip = this.data.vip ?? false;
    const item = await this.getItemMaster();
    const queueNumber = await this.bucketService.getQueue(
      this.data.item_id,
      vip,
    );
    const prefix = vip ? 'B' : 'A';
    const code = `${prefix}${padCode(queueNumber)}`;

    Object.assign(this.data, {
      status: STATUS.WAITING,
      time: new Date().getTime(),
      item_queue_id: item.item_queue_id,
      vip,
      code,
    });
    return;
  }

  async getItemMaster(): Promise<ItemModel> {
    const queueItem: QueueItemModel = await this.dataService.item.findOne({
      relations: ['item'],
      where: {
        id: this.data.item_id,
      },
    });

    return queueItem.item;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  get validateRelations(): validateRelations[] {
    return [];
  }

  get uniqueColumns(): columnUniques[] {
    return [];
  }

  get eventTopics(): EventTopics[] {
    return [];
  }

  get entityTarget(): any {
    return QueueModel;
  }
}
